from typing import Generator

from sqlmodel import create_engine, SQLModel, Session, text, SQLModel, MetaData
from sqlalchemy.orm import registry

FLOORS_DATABASE_URL = "sqlite:///./fost.gestion_studio.floors.db"

floors_engine = create_engine(FLOORS_DATABASE_URL, echo=False)
floors_registry = registry()


class FloorsSQLModel(SQLModel, registry=floors_registry):
    pass


def create_db_and_tables():
    FloorsSQLModel.metadata.create_all(floors_engine)
    if floors_engine.url.drivername.lower().startswith("sqlite://"):
        with floors_engine.connect() as connection:
            connection.execute(text("PRAGMA foreign_keys=ON"))  # for SQLite only


def get_session() -> Generator[Session, None, None]:
    with Session(floors_engine) as session:
        yield session



from __future__ import annotations

import uuid
from typing import List, Optional
from pydantic import (
    EmailStr,
    BaseModel,
    Field,
)


from ..models import UserBase, ProjectBase, TaskBase, TaskStatus


# --- User Schemas ---


class UserCreate(UserBase):
    pass


class UserRead(UserBase):
    id: uuid.UUID


class UserUpdate(BaseModel):
    login: Optional[str] = None
    email: Optional[EmailStr] = None


# --- Project Schemas ---


class ProjectCreate(ProjectBase):
    pass


class ProjectRead(ProjectBase):
    id: uuid.UUID


class ProjectUpdate(BaseModel):
    code: Optional[str] = None
    title: Optional[str] = None


# --- Task Schemas ---


class TaskCreate(TaskBase):
    pass


class TaskReadBase(
    TaskBase
):  # A base for TaskRead to avoid direct recursion if TaskRead needs UserRead etc.
    id: uuid.UUID


class TaskUpdate(BaseModel):
    title: Optional[str] = None
    description: Optional[str] = None  # Added description here
    status: Optional[TaskStatus] = None
    assignee_id: Optional[uuid.UUID] = Field(
        default=None,
        nullable=True,
        description="Set to null or omit to unassign/leave as is",
    )


# --- Schemas with Relationships (for responses) ---


class UserReadWithProjects(UserRead):
    projects: List[ProjectRead] = []


class ProjectReadWithUsers(ProjectRead):
    users: List[UserRead] = []


class TaskRead(TaskReadBase):
    assignee: Optional[UserRead] = None
    project: ProjectRead


class ProjectReadWithDetails(ProjectRead):
    users: List[UserRead] = []
    tasks: List[TaskRead] = []


# Pydantic v2 automatically calls model_rebuild() on modules,
# but if you encounter issues with forward references not resolving,
# you can explicitly update them for each model at the end of the file.
# This is typically not needed for simple string forward refs within the same module.
# For Pydantic v1, you would do:
# UserReadWithProjects.update_forward_refs()
# ProjectReadWithUsers.update_forward_refs()
# TaskRead.update_forward_refs()
# ProjectReadWithDetails.update_forward_refs()

# For Pydantic V2, models are automatically rebuilt.
# If you had models in different files that depended on each other, you might need
# to ensure they are all imported and then call model_rebuild() on them.
# But within a single schemas.py, it should work out of the box.



import uuid
from typing import List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlmodel import Session, select, col
from sqlalchemy.exc import IntegrityError

from ..database import get_session
from ..models import (
    User as DBUser,
    Project as DBProject,
    ProjectUsers as DBProjectUsers,
    Task as DBTask,
)

# Import Pydantic schemas from the new schemas.py
from .schemas import UserRead, UserCreate, UserUpdate, UserReadWithProjects, ProjectRead

router = APIRouter(
    prefix="/users",
    tags=["Projects Users"],
)

# --- Pydantic models are now in routers/schemas.py ---


@router.post("/", response_model=UserRead, status_code=status.HTTP_201_CREATED)
def create_user(
    *, session: Session = Depends(get_session), user_in: UserCreate
) -> DBUser:
    db_user = DBUser.model_validate(user_in)
    try:
        session.add(db_user)
        session.commit()
        session.refresh(db_user)
        return db_user
    except IntegrityError as e:
        session.rollback()
        if "user.login" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_409_CONFLICT, detail="Login already exists"
            )
        elif "user.email" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_409_CONFLICT, detail="Email already exists"
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=f"Database integrity error: {e}",
            )


@router.get("/", response_model=List[UserReadWithProjects])
def read_users(
    *, session: Session = Depends(get_session), skip: int = 0, limit: int = 100
) -> List[DBUser]:
    users = session.exec(select(DBUser).offset(skip).limit(limit)).all()
    return users


@router.get("/{user_id}", response_model=UserReadWithProjects)
def read_user(
    *, session: Session = Depends(get_session), user_id: uuid.UUID
):  # Return type will be UserReadWithProjects
    db_user = session.get(DBUser, user_id)
    if not db_user:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="User not found"
        )
    return db_user
    # project_links = session.exec(
    #     select(DBProjectUsers).where(DBProjectUsers.user_id == user_id)
    # ).all()

    # project_ids = [link.project_id for link in project_links]
    # user_projects_data = []
    # if project_ids:
    #     user_projects_db = session.exec(
    #         select(DBProject).where(col(DBProject.id).in_(project_ids))
    #     ).all()
    #     user_projects_data = [
    #         ProjectRead.model_validate(p) for p in user_projects_db
    #     ]  # Use ProjectRead from schemas

    # # Construct the response model using UserRead and the fetched projects
    # user_data = UserRead.model_validate(db_user).model_dump()  # Get core user fields

    # # Create UserReadWithProjects instance
    # return UserReadWithProjects(**user_data, projects=user_projects_data)


@router.patch("/{user_id}", response_model=UserRead)
def update_user(
    *, session: Session = Depends(get_session), user_id: uuid.UUID, user_in: UserUpdate
) -> DBUser:
    db_user = session.get(DBUser, user_id)
    if not db_user:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="User not found"
        )

    user_data = user_in.model_dump(exclude_unset=True)  # UserUpdate is SQLModel based
    for key, value in user_data.items():
        setattr(db_user, key, value)

    try:
        session.add(db_user)
        session.commit()
        session.refresh(db_user)
        return db_user
    except IntegrityError as e:
        session.rollback()
        if "user.login" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_409_CONFLICT, detail="Login already exists"
            )
        elif "user.email" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_409_CONFLICT, detail="Email already exists"
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=f"Database integrity error: {e}",
            )


@router.delete("/{user_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_user(*, session: Session = Depends(get_session), user_id: uuid.UUID):
    db_user = session.get(DBUser, user_id)
    if not db_user:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="User not found"
        )

    links_to_delete = session.exec(
        select(DBProjectUsers).where(DBProjectUsers.user_id == user_id)
    ).all()
    for link in links_to_delete:
        session.delete(link)

    tasks_to_update = session.exec(
        select(DBTask).where(DBTask.assignee_id == user_id)
    ).all()
    for task in tasks_to_update:
        task.assignee_id = None
        session.add(task)

    session.delete(db_user)
    session.commit()
    return



from nicegui import ui
from fastapi import APIRouter

from ...api import ProjectsAPI, models
from ....components.header import header

PROJECTS_API_CLIENT = None


pages_router = APIRouter()


def project_api() -> ProjectsAPI:
    assert PROJECTS_API_CLIENT is not None  # Initialized by mount_pages()
    return PROJECTS_API_CLIENT


@ui.page(path="/", api_router=pages_router)
def home():
    header("Projects")
    # apply_colors()
    # with ui.header(elevated=True):
    #     with ui.row(align_items="center w-full"):
    #         with ui.link(target="/"):
    #             ui.button(icon="sym_o_home")
    #         ui.label("Projects").classes("text-lg")
    #         ui.space()
    #         dark_mode_switch()
    #     # with ui.row(align_items="center w-full"):
    #     #     with ui.link(target="/"):
    #     #         ui.button(icon="sym_o_arrow_circle_left")
    #     #     ui.markdown("## Projects")

    with ui.grid(columns=3).classes("gap-2"):
        for i in range(30):
            ui.label(f"Project #{i}").classes("border grow")
            ui.label(f"WIP").classes("border")
            with ui.link(target="admin").classes("border"):
                ui.icon("sym_o_mystery", size="md")  # , color="stone-700")
                # ui.icon("eye")


def mount_pages(parent_router: APIRouter, api_host: str, api_port: int):
    project_api = ProjectsAPI(host=api_host, port=api_port)
    parent_router.include_router(pages_router)



from nicegui import ui

from ..components.header import header
from ..components.link_lists import big_button_links

from ..settings import PyFostSettings

settings = PyFostSettings()


def home_page():
    header("Gestion Studio", None)
    pages = [
        [
            ("Floor Plans", "sym_o_explore", "apps/floors/"),
            ("Projects", "sym_o_interactive_space", "apps/projects/"),
        ],
        [
            ("Settings", "sym_o_handyman", ""),
            ("API Doc", "auto_fix_high", "/docs"),
        ],
    ]
    big_button_links(pages)



import uuid
from typing import List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlmodel import Session, select, col
from sqlalchemy.exc import IntegrityError

from ..database import get_session
from ..models import (
    FacilityBase,
    Facility,
    FloorBase,
    Floor,
    SeatBase,
    Seat,
)

router = APIRouter(
    tags=["Floors Seats"],
)

#
# ---- Facility
#


@router.post(
    "/facilities/", response_model=Facility, status_code=status.HTTP_201_CREATED
)
def create_facility(
    *, session: Session = Depends(get_session), facility_in: FacilityBase
) -> Facility:
    new_one = Facility.model_validate(facility_in)
    try:
        session.add(new_one)
        session.commit()
        session.refresh(new_one)
        return new_one
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.get("/facilities/", response_model=List[Facility])
def read_facilities(
    *, session: Session = Depends(get_session), skip: int = 0, limit: int = 100
) -> List[Facility]:
    found = session.exec(select(Facility).offset(skip).limit(limit)).all()
    return found


@router.get("/facilities/{facility_id}", response_model=Facility)
def read_facility(*, session: Session = Depends(get_session), facility_id: uuid.UUID):
    found_one = session.get(Facility, facility_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Facility not found"
        )
    return found_one


@router.patch("/facilities/{facility_id}", response_model=Facility)
def update_facility(
    *,
    session: Session = Depends(get_session),
    facility_id: uuid.UUID,
    facility_in: FacilityBase,
) -> Facility:
    found_one = session.get(Facility, facility_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Facility not found"
        )

    data = facility_in.model_dump(exclude_unset=True)
    for key, value in data.items():
        setattr(found_one, key, value)

    try:
        session.add(found_one)
        session.commit()
        session.refresh(found_one)
        return found_one
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.delete("/facilities/{facility_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_facility(*, session: Session = Depends(get_session), facility_id: uuid.UUID):
    found_one = session.get(Facility, facility_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Facility not found"
        )

    # FIXME: Deal with related stuff (assignment, etc...)

    session.delete(found_one)
    session.commit()


#
# ---- Floor
#


@router.post("/floors/", response_model=Floor, status_code=status.HTTP_201_CREATED)
def create_floor(
    *, session: Session = Depends(get_session), floor_in: FloorBase
) -> Floor:
    new_one = Floor.model_validate(floor_in)
    try:
        session.add(new_one)
        session.commit()
        session.refresh(new_one)
        return new_one
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.get("/floors/", response_model=List[Floor])
def read_floors(
    *, session: Session = Depends(get_session), skip: int = 0, limit: int = 100
) -> List[Floor]:
    found = session.exec(select(Floor).offset(skip).limit(limit)).all()
    return found


@router.get("/floors/{floor_id}", response_model=Floor)
def read_floor(*, session: Session = Depends(get_session), floor_id: uuid.UUID):
    found_one = session.get(Floor, floor_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Floor not found"
        )
    return found_one


@router.patch("/floors/{floor_id}", response_model=Floor)
def update_floor(
    *,
    session: Session = Depends(get_session),
    floor_id: uuid.UUID,
    floor_in: FacilityBase,
) -> Facility:
    found_one = session.get(Floor, floor_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Floor not found"
        )

    data = floor_in.model_dump(exclude_unset=True)
    for key, value in data.items():
        setattr(found_one, key, value)

    try:
        session.add(found_one)
        session.commit()
        session.refresh(found_one)
        return found_one
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.delete("/floors/{floor_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_floor(*, session: Session = Depends(get_session), floor_id: uuid.UUID):
    found_one = session.get(Floor, floor_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Floor not found"
        )

    # FIXME: Deal with related stuff (assignment, etc...)

    session.delete(found_one)
    session.commit()


#
# ---- Seat
#


@router.post("/seats/", response_model=Seat, status_code=status.HTTP_201_CREATED)
def create_floor(*, session: Session = Depends(get_session), seat_in: SeatBase) -> Seat:
    new_one = Seat.model_validate(seat_in)
    try:
        session.add(new_one)
        session.commit()
        session.refresh(new_one)
        return new_one
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.get("/seats/", response_model=List[Seat])
def read_seats(
    *, session: Session = Depends(get_session), skip: int = 0, limit: int = 100
) -> List[Floor]:
    found = session.exec(select(Seat).offset(skip).limit(limit)).all()
    return found


@router.get("/seats/{seat_id}", response_model=Seat)
def read_seat(*, session: Session = Depends(get_session), seat_id: uuid.UUID):
    found_one = session.get(Seat, seat_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Seat not found"
        )
    return found_one


@router.patch("/seats/{seat_id}", response_model=Seat)
def update_seat(
    *,
    session: Session = Depends(get_session),
    seat_id: uuid.UUID,
    seat_in: FacilityBase,
) -> Facility:
    found_one = session.get(Seat, seat_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Seat not found"
        )

    data = seat_in.model_dump(exclude_unset=True)
    for key, value in data.items():
        setattr(found_one, key, value)

    try:
        session.add(found_one)
        session.commit()
        session.refresh(found_one)
        return found_one
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.delete("/seats/{seta_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_seat(*, session: Session = Depends(get_session), seat_id: uuid.UUID):
    found_one = session.get(Floor, seat_id)
    if not found_one:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Seat not found"
        )

    # FIXME: Deal with related stuff (assignment, etc...)

    session.delete(found_one)
    session.commit()



from contextlib import asynccontextmanager

from fastapi import FastAPI, APIRouter
from nicegui import ui

from ..settings import PyFostSettings

from .page import home_page


async def startup(app: FastAPI):
    print("INFO:     Starting up Home Frontend")


async def teardown(app: FastAPI):
    print("INFO:     Shutting down Home Frontend...")


def mount(app: FastAPI, api_host: str, api_port: int):
    ui.page("/")(home_page)


@asynccontextmanager
async def lifespan(app: FastAPI):
    yield


app = FastAPI(
    title="Home",
    description="PyFost Home",
    version="0.1.0",
    lifespan=lifespan,
)

settings = PyFostSettings().apps.home_frontend

mount(
    app,
    settings.api_host,
    settings.api_port,
)

if __name__ == "__main__":
    import uvicorn

    uvicorn.run(
        "pyfost.gestion_studio.home.main:app",
        host="0.0.0.0",
        port=8004,
        reload=True,
    )



import os
import httpx
from typing import Any

from .models import Facility, Floor, Seat, Resource, Assignation, Task


class FloorsAPI:
    def __init__(self, host: str | None = None, port: int | None = None):
        host = host or os.environ["PYFOST__APPS__FLOORS__HOST"]
        port = port or int(os.environ["PYFOST__APPS__FLOORS__PORT"])
        url = f"http://{host}:{port}/api/floors/"

        self._client = httpx.AsyncClient(base_url=url, timeout=2.0)

    async def _request(self, method: str, endpoint: str, **kwargs) -> Any:
        """Helper function to make requests and handle basic errors."""
        try:
            response = await self._client.request(method, endpoint, **kwargs)
            response.raise_for_status()  # Raise an exception for bad status codes (4xx or 5xx)
            if response.status_code == 204:  # No Content
                return None
            return response.json()
        except httpx.HTTPStatusError as e:
            # Try to get detail from response, otherwise use generic message
            detail = (
                e.response.json().get("detail", e.response.text)
                if e.response
                else str(e)
            )
            print(
                f"HTTP Error: {e.response.status_code} - {detail}"
            )  # Log detailed error
            raise Exception(
                f"API Error on {self._client.base_url}{endpoint} ({e.response.status_code}): {detail}"
            ) from e
        except httpx.RequestError as e:
            print(f"Request Error: {e}")
            raise Exception(f"Network or connection error: {e}") from e
        except Exception as e:
            print(f"Unexpected Error during API call: {e}")
            raise Exception(
                "An unexpected error occurred while contacting the API."
            ) from e

    async def get_facilities(self) -> list[Facility]:
        return [Facility(**i) for i in await self._request("GET", "facilities/")]

    async def get_floors(self) -> list[Floor]:
        return [Floor(**i) for i in await self._request("GET", "floors/")]

    async def get_seats(self) -> list[Seat]:
        return [Seat(**i) for i in await self._request("GET", "seats/")]

    async def get_resources(self) -> list[Resource]:
        return [Resource(**i) for i in await self._request("GET", "resources/")]

    async def get_assignations(self) -> list[Assignation]:
        return [Assignation(**i) for i in await self._request("GET", "assignations/")]

    async def get_tasks(self) -> list[Task]:
        return [Task(**i) for i in await self._request("GET", "tasks/")]



from contextlib import asynccontextmanager

from fastapi import FastAPI, APIRouter

from ...settings import PyFostSettings
from .admin import mount_admin
from .pages import mount_pages


async def startup(app: FastAPI):
    print("INFO:     Starting up Floors Frontend")


async def teardown(app: FastAPI):
    print("INFO:     Shutting down Floors Frontend...")


def mount(app: FastAPI, api_host: str, api_port: int):
    floors_app_frontend_router = APIRouter(prefix="/apps/floors")
    mount_pages(floors_app_frontend_router, api_host, api_port)
    mount_admin(floors_app_frontend_router, api_host, api_port)
    app.include_router(floors_app_frontend_router)


@asynccontextmanager
async def lifespan(app: FastAPI):
    yield


app = FastAPI(
    title="Floors Management Frontend",
    description="GUI for managing Facilities, Floors, Seats, Resources and Assigments.",
    version="0.1.0",
    lifespan=lifespan,
)

settings = PyFostSettings().apps.projects_frontend

mount(
    app,
    settings.api_host,
    settings.api_port,
)

if __name__ == "__main__":
    import uvicorn

    uvicorn.run(
        "pyfost.gestion_studio.floors.frontend.main:app",
        host="0.0.0.0",
        port=8004,
        reload=True,
    )



import uuid
from typing import List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlmodel import Session, select, col
from sqlalchemy.exc import IntegrityError

from ..database import get_session
from ..models import (
    Project as DBProject,
    User as DBUser,
    ProjectUsers as DBProjectUsers,
    Task as DBTask,
)

# Import Pydantic schemas
from .schemas import (
    ProjectRead,
    ProjectCreate,
    ProjectUpdate,
    ProjectReadWithUsers,
    ProjectReadWithDetails,
    UserRead,
)

router = APIRouter(
    prefix="/projects",
    tags=["Projects"],
)


@router.post("/", response_model=ProjectRead, status_code=status.HTTP_201_CREATED)
def create_project(
    *, session: Session = Depends(get_session), project_in: ProjectCreate
) -> DBProject:
    db_project = DBProject.model_validate(project_in)
    try:
        session.add(db_project)
        session.commit()
        session.refresh(db_project)
        return db_project
    except IntegrityError as e:
        session.rollback()
        if "project.code" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_409_CONFLICT,
                detail="Project code already exists",
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=f"Database integrity error: {e}",
            )


@router.get("/", response_model=List[ProjectRead])
def read_projects(
    *, session: Session = Depends(get_session), skip: int = 0, limit: int = 100
) -> List[DBProject]:
    projects = session.exec(select(DBProject).offset(skip).limit(limit)).all()
    return projects


@router.get("/{project_id}", response_model=ProjectReadWithDetails)
def read_project(*, session: Session = Depends(get_session), project_id: uuid.UUID):
    db_project = session.get(DBProject, project_id)
    if not db_project:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Project not found"
        )
    return db_project

    # # Fetch users
    # user_links = session.exec(
    #     select(DBProjectUsers).where(DBProjectUsers.project_id == project_id)
    # ).all()
    # user_ids = [link.user_id for link in user_links]
    # project_users_data = []
    # if user_ids:
    #     project_users_db = session.exec(
    #         select(DBUser).where(col(DBUser.id).in_(user_ids))
    #     ).all()
    #     project_users_data = [UserRead.model_validate(u) for u in project_users_db]

    # # Fetch tasks
    # project_tasks_db = session.exec(
    #     select(DBTask).where(DBTask.project_id == project_id)
    # ).all()
    # # We need to convert DBTask to schemas.TaskRead
    # project_tasks_data = []
    # for db_task_item in project_tasks_db:
    #     task_item_data = db_task_item.model_dump()
    #     if db_task_item.assignee:
    #         task_item_data["assignee"] = UserRead.model_validate(db_task_item.assignee)
    #     else:
    #         task_item_data["assignee"] = None
    #     # The project part of TaskRead is just ProjectRead, not ProjectReadWithDetails (to avoid recursion)
    #     # The current db_project is the project for these tasks
    #     task_item_data["project"] = ProjectRead.model_validate(
    #         db_project
    #     )  # or fetch project if needed for each task differently
    #     project_tasks_data.append(TaskRead(**task_item_data))

    # project_data = ProjectRead.model_validate(db_project).model_dump()
    # return ProjectReadWithDetails(
    #     **project_data, users=project_users_data, tasks=project_tasks_data
    # )


@router.patch("/{project_id}", response_model=ProjectRead)
def update_project(
    *,
    session: Session = Depends(get_session),
    project_id: uuid.UUID,
    project_in: ProjectUpdate,
) -> DBProject:
    db_project = session.get(DBProject, project_id)
    if not db_project:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Project not found"
        )

    project_data_update = project_in.model_dump(exclude_unset=True)
    for key, value in project_data_update.items():
        setattr(db_project, key, value)

    try:
        session.add(db_project)
        session.commit()
        session.refresh(db_project)
        return db_project
    except IntegrityError as e:
        session.rollback()
        if "project.code" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_409_CONFLICT,
                detail="Project code already exists on another project",
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=f"Database integrity error: {e}",
            )


@router.delete("/{project_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_project(*, session: Session = Depends(get_session), project_id: uuid.UUID):
    db_project = session.get(DBProject, project_id)
    if not db_project:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Project not found"
        )

    # tasks_to_delete = session.exec(
    #     select(DBTask).where(DBTask.project_id == project_id)
    # ).all()
    # for task in tasks_to_delete:
    #     session.delete(task)

    # links_to_delete = session.exec(
    #     select(DBProjectUsers).where(DBProjectUsers.project_id == project_id)
    # ).all()
    # for link in links_to_delete:
    #     session.delete(link)

    session.delete(db_project)
    session.commit()


@router.post(
    "/{project_id}/users/{user_id}",
    # response_model=ProjectReadWithUsers,
    # tags=["Project User Management"],
)
def add_user_to_project(
    *,
    session: Session = Depends(get_session),
    project_id: uuid.UUID,
    user_id: uuid.UUID,
):
    existing_link = session.exec(
        select(DBProjectUsers).where(
            DBProjectUsers.project_id == project_id, DBProjectUsers.user_id == user_id
        )
    ).first()

    if existing_link:
        raise HTTPException(
            status_code=status.HTTP_409_CONFLICT, detail="User already in project"
        )

    db_project = session.get(DBProject, project_id)
    if not db_project:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Project not found"
        )

    db_user = session.get(DBUser, user_id)
    if not db_user:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="User not found"
        )

    new_link = DBProjectUsers(project_id=project_id, user_id=user_id)
    session.add(new_link)
    session.commit()


@router.delete(
    "/{project_id}/users/{user_id}",
    response_model=ProjectReadWithUsers,
    # tags=["Project User Management"],
)
def remove_user_from_project(
    *,
    session: Session = Depends(get_session),
    project_id: uuid.UUID,
    user_id: uuid.UUID,
):  # Return type is ProjectReadWithUsers
    db_project = session.get(DBProject, project_id)
    if not db_project:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Project not found"
        )

    db_user = session.get(DBUser, user_id)
    if not db_user:  # Check user existence for better error message
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="User not found"
        )

    link_to_delete = session.exec(
        select(DBProjectUsers).where(
            DBProjectUsers.project_id == project_id, DBProjectUsers.user_id == user_id
        )
    ).first()

    if not link_to_delete:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="User not associated with this project",
        )

    session.delete(link_to_delete)
    session.commit()

    # Reconstruct ProjectReadWithUsers response
    user_links = session.exec(
        select(DBProjectUsers).where(DBProjectUsers.project_id == project_id)
    ).all()
    current_user_ids = [link.user_id for link in user_links]
    project_users_data = []
    if current_user_ids:
        project_users_db = session.exec(
            select(DBUser).where(col(DBUser.id).in_(current_user_ids))
        ).all()
        project_users_data = [UserRead.model_validate(u) for u in project_users_db]

    project_response_data = ProjectRead.model_validate(db_project).model_dump()
    return ProjectReadWithUsers(**project_response_data, users=project_users_data)



from __future__ import annotations

# This duplicates all the query and response models from the backend.
# Which sucks.
# But in real world situation, this api client would be generated from
# the backend openapi specs, and all would be duplicated.
# So I guess that's ok.
# (and we should generate this api from the backend openapi!!!)

import enum
import uuid
import datetime

from pydantic import BaseModel, Field, EmailStr, UUID4


class FloorsModel(BaseModel):
    pass


class Facility(FloorsModel):
    id: uuid.UUID | None
    name: str
    floors: list[Floor]


class Floor(FloorsModel):
    id: uuid.UUID | None
    name: str
    facility: Facility
    seats: list[Seat]


class Seat(FloorsModel):
    id: uuid.UUID | None
    code: str
    position_x: int
    position_y: int

    floor: Floor
    assignations: list[Assignation]


class ResourceCategory(str, enum.Enum):
    USER = "User"
    WORKSTATION = "Workstation"
    SOFTWARE_LICENSE = "SoftwareLicense"
    OTHER = "Other"


class Resource(FloorsModel):
    id: uuid.UUID | None
    category: ResourceCategory
    name: str
    properties: str  # JsonValue
    assignations: list[Assignation]


class AssignationStatus(str, enum.Enum):
    PLANNED = "Planned"
    WIP = "In Progress"
    READY = "Ready"


class Assignation(FloorsModel):
    id: uuid.UUID | None
    start_date: datetime.date
    end_date: datetime.date
    status: AssignationStatus
    seat: Seat
    resource: Resource
    tasks: list[Task]


class TaskType(str, enum.Enum):
    SETUP_WORKSTATION = "Setup Workstation"
    INSTALL_SOFTWARES = "Install Softwares"


class TaskStatus(str, enum.Enum):
    PENDING = "Pending"
    IN_PROGRESS = "In Progress"
    COMPLETED = "Completed"
    BLOCKED = "Blocked"


class Task(FloorsModel):
    id: uuid.UUID | None
    type: TaskType
    status: TaskStatus
    notes: str
    assignation_id: uuid.UUID | None
    assignation: Assignation



from .home import HomeFrontend
from .floors import FloorsFrontend
from .projects import ProjectsFrontend

from ._app import create_app, run_app

from .settings import PyFostSettings

settings = PyFostSettings()
runtime_settings = settings.runtime.frontends

app = create_app(
    title=runtime_settings.title,
    description=runtime_settings.description,
    version=runtime_settings.version,
    apps=(
        HomeFrontend(),
        FloorsFrontend(),
        ProjectsFrontend(),
    ),
)

if __name__ == "__main__":
    run_app(
        "pyfost.gestion_studio.main_frontends:app",
        host=runtime_settings.host,
        port=runtime_settings.port,
        reload=runtime_settings.reload,
    )



"""
Use this to spawn a standalone Projects service.

"""

from fastapi import FastAPI
from contextlib import asynccontextmanager

from .database import create_db_and_tables
from .routers import assignations, resources, seats
from ...settings import PyFostSettings


async def startup(app: FastAPI):
    print("INFO:     Starting up Floors API and creating database tables...")
    create_db_and_tables()


async def teardown(app: FastAPI):
    print("INFO:     Shutting down Floors API...")


def mount(app: FastAPI):
    app.include_router(assignations.router, prefix="/api/floors")
    app.include_router(resources.router, prefix="/api/floors")
    app.include_router(seats.router, prefix="/api/floors")


@asynccontextmanager
async def lifespan(app: FastAPI):
    await startup(app)
    yield
    await teardown(app)


settings = PyFostSettings().apps.projects_backend

app = FastAPI(
    title=settings.title,
    description=settings.description,
    version=settings.version,
    lifespan=lifespan,
)

mount(app)


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(
        "pyfost.gestion_studio.floors.backend.main:app",
        host=settings.host,
        port=settings.port,
        reload=settings.reload,
    )



from __future__ import annotations


from nicegui import ui

from pyfost.adminui import Admin, ModelRenderer

from ...api import ProjectsAPI, models
from ....components.header import header


class UserRenderer(ModelRenderer):
    @classmethod
    def cell_email(self, model, admin):
        return str(model.email)

    @classmethod
    def cell_projects(self, model, admin):
        return ",".join([i.code for i in model.projects])

    @classmethod
    def render_login(cls, model, admin):
        ui.button(model.login)
        # with ui.row():
        #     for t in model.tags:
        #         ui.button(
        #             t.name,
        #             on_click=lambda t=t: ui.navigate.to(
        #                 f"{admin.prefix}/{t.__class__.__name__}/{t.name}"
        #             ),
        #         ).props("flat")


class TaskRenderer(ModelRenderer):
    @classmethod
    def cell_project(self, model, admin):
        return model.project.title

    @classmethod
    def cell_status(self, model, admin):
        return model.status.value

    @classmethod
    def cell_assignee(self, model, admin):
        assignee = model.assignee
        if assignee is None:
            return "<FREE>"
        return assignee.login


def mount_admin(parent_router, api_host: str, api_port: int):
    project_api = ProjectsAPI(host=api_host, port=api_port)

    async def get_users() -> list[models.User]:
        return await project_api.get_users()

    async def get_projects() -> list[models.Project]:
        return await project_api.get_projects()

    async def get_tasks() -> list[models.Task]:
        return await project_api.get_tasks()

    def admin_header():
        header(
            "Projects Admin",
            back_target=parent_router.prefix + "/",
            back_icon="sym_o_arrow_circle_left",
        )

    admin = Admin("/admin")
    admin.title = "Projects Admin"
    admin.set_header(admin_header)
    admin.add_view(models.User, get_users, renderer=UserRenderer(pk_name="id"))
    admin.add_view(models.Project, get_projects)
    admin.add_view(models.Task, get_tasks, renderer=TaskRenderer())
    admin.add_to_router(parent_router)



import uuid
from typing import List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlmodel import Session, select, col
from sqlalchemy.exc import IntegrityError

from ..database import get_session
from ..models import (
    AssignationBase,
    Assignation,
    AssignationStatus,
)

router = APIRouter(
    prefix="/assignations",
    tags=["Floors Assignations"],
)



from .floors import FloorsBackend
from .projects import ProjectsBackend

from ._app import create_app, run_app

from .settings import PyFostSettings

settings = PyFostSettings()
runtime_settings = settings.runtime.backends

app = create_app(
    title=runtime_settings.title,
    description=runtime_settings.description,
    version=runtime_settings.version,
    apps=(
        FloorsBackend(),
        ProjectsBackend(),
    ),
)

if __name__ == "__main__":
    run_app(
        "pyfost.gestion_studio.main_backends:app",
        host=runtime_settings.host,
        port=runtime_settings.port,
        reload=runtime_settings.reload,
    )



import os

from .home import HomeFrontend
from .floors import FloorsBackend, FloorsFrontend
from .projects import ProjectsBackend, ProjectsFrontend

from ._app import create_app, run_app

from .settings import PyFostSettings

settings = PyFostSettings()
runtime_settings = settings.runtime.all

app = create_app(
    title=runtime_settings.title,
    description=runtime_settings.description,
    version=runtime_settings.version,
    apps=(
        HomeFrontend(settings=runtime_settings),
        FloorsBackend(settings=runtime_settings),
        FloorsFrontend(settings=runtime_settings),
        ProjectsBackend(settings=runtime_settings),
        ProjectsFrontend(settings=runtime_settings),
    ),
)

if __name__ == "__main__":
    run_app(
        "pyfost.gestion_studio.main_all:app",
        host=runtime_settings.host,
        port=runtime_settings.port,
        reload=runtime_settings.reload,
    )



#!!! YOU CAN USE THIS WITH SQLModels: from __future__ import annotations

import uuid
from typing import List, Optional, TYPE_CHECKING
from enum import Enum

from sqlmodel import Field, SQLModel, Relationship
from pydantic import EmailStr

# TYPE_CHECKING block in models.py is primarily for forward references
# between SQLModel table classes themselves, if needed.
# Pydantic schema forward references are now handled within routers/schemas.py.
if TYPE_CHECKING:
    # These are SQLModel classes, not Pydantic schemas
    # pass # Usually not strictly needed if using string names in Relationship
    # For example, if User had a direct reference to Project SQLModel class
    # and vice-versa, this block would be crucial.
    # With the link table, direct User <-> Project SQLModel class refs are gone.
    # Task.assignee: Optional["User"] works because "User" is defined later in the file
    # or Python resolves it.
    pass


# --- Link Table: ProjectUsers ---
class ProjectUsers(SQLModel, table=True):
    __tablename__ = "project_users"

    project_id: uuid.UUID | None = Field(
        default=None, foreign_key="project.id", primary_key=True
    )
    user_id: uuid.UUID | None = Field(
        default=None, foreign_key="user.id", primary_key=True
    )

    # project: Optional[Project] = Relationship(back_populates="user_links")
    # user: Optional[User] = Relationship(back_populates="project_links")


# --- Task Status Enum ---
class TaskStatus(str, Enum):
    NYS = "NYS"
    WIP = "WIP"
    DONE = "Done"


# --- User Model ---


class UserBase(SQLModel):
    login: str = Field(unique=True, index=True)
    email: EmailStr = Field(unique=True, index=True)


class User(UserBase, table=True):
    id: Optional[uuid.UUID] = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )

    projects: list["Project"] = Relationship(
        back_populates="users", link_model=ProjectUsers
    )
    assigned_tasks: list["Task"] = Relationship(back_populates="assignee")


# --- Project Model ---
class ProjectBase(SQLModel):
    code: str = Field(unique=True, index=True)
    title: str


class Project(ProjectBase, table=True):
    id: Optional[uuid.UUID] = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )

    users: list["User"] = Relationship(
        back_populates="projects", link_model=ProjectUsers
    )
    tasks: List["Task"] = Relationship(back_populates="project", cascade_delete=True)


# --- Task Model ---
class TaskBase(SQLModel):
    title: str
    description: Optional[str] = Field(default=None)
    status: TaskStatus = Field(default=TaskStatus.NYS)

    assignee_id: Optional[uuid.UUID] = Field(
        default=None, foreign_key="user.id", index=True, nullable=True
    )
    project_id: uuid.UUID = Field(foreign_key="project.id", index=True)


class Task(TaskBase, table=True):
    id: Optional[uuid.UUID] = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )

    assignee: Optional[User] = Relationship(back_populates="assigned_tasks")
    project: Project = Relationship(back_populates="tasks")



from nicegui import ui

from .theming import apply_colors, dark_mode_switch


def header(title: str, back_target: str = "/", back_icon: str = "sym_o_home"):
    apply_colors()

    with ui.header(elevated=True):
        with ui.row(align_items="center w-full").classes("xgap-0"):
            if back_target is not None:
                with ui.link(target=back_target):
                    ui.button(icon=back_icon).props("fab").classes("fg-secondary")
            with ui.row(align_items="baseline"):
                ui.image("/assets/fost_studio_logo_noborders.jpeg").classes("w-16")
                ui.label(title).classes("text-h4 text-weight-bolder")
            ui.space()
            dark_mode_switch()



from typing import TypeVar
from contextlib import asynccontextmanager

from pydantic import BaseModel, IPvAnyNetwork, AnyUrl

from fastapi import FastAPI
from nicegui import ui
import uvicorn

from ._version import version

pyfost_getsion_studio_version = version

# --- Settings Models


class AppSettings(BaseModel):
    title: str
    description: str
    version: str = pyfost_getsion_studio_version
    host: IPvAnyNetwork = "127.0.0.1"
    port: int = 8001
    reload: bool = True


class AppBackendSettings(AppSettings):
    database_url: AnyUrl | None = None


class AppFrontendSettings(AppSettings):
    api_host: IPvAnyNetwork = "127.0.0.1"
    api_port: int = 8001
    nice_secret: str = "THIS_IS_NOT_A_SAFE_VALUE"


class AppAllSettings(AppBackendSettings, AppFrontendSettings):
    pass


# --- App Types
AppSettingsType = TypeVar("AppSettingsType")


class _App[AppSettingsType]:
    def __init__(self, settings: AppSettingsType):
        self.settings = settings

    async def startup(self, app: FastAPI) -> None:
        pass

    async def teardown(self, app: FastAPI) -> None:
        pass

    def mount(self, app: FastAPI) -> None:
        pass


class AppBackend(_App[AppBackendSettings]):
    pass


class AppFrontend(_App[AppFrontendSettings]):
    def __init__(self, settings):
        super().__init__(settings)

        from pathlib import Path
        from .. import gestion_studio
        import nicegui.app

        assets_path = Path(gestion_studio.__file__) / ".." / "assets"
        assets_path = assets_path.resolve()
        print("========>", assets_path)
        nicegui.app.add_media_files("/assets", assets_path)


class AppAll(_App[AppAllSettings]):
    pass


def create_app(
    title,
    description,
    version,
    apps: list[_App],
):
    @asynccontextmanager
    async def lifespan(fastapi_app: FastAPI):
        for app in apps:
            await app.startup(fastapi_app)
        yield
        for app in apps:
            await app.teardown(fastapi_app)

    fastapi_app = FastAPI(
        title=title,
        description=description,
        version=version,
        lifespan=lifespan,
    )

    for app in apps:
        app.mount(fastapi_app)

    ui.run_with(
        fastapi_app,
        # mount_path=f"/ui",
        title=title,  # Default title for pages without specific one
        storage_secret="storage_secret",  # Needed for session management FIXME: get it from args
        # Add other ui.run options if needed (e.g., favicon)
    )
    return fastapi_app


def run_app(
    app_object_path: str,
    host: str = "0.0.0.0",
    port: int = 8081,
    reload: bool = True,
):
    uvicorn.run(
        # "pyfost.gestion_studio.projects.backend.main:app",
        app_object_path,
        host=host,
        port=port,
        reload=reload,
    )



from nicegui import ui
from fastapi import APIRouter

# from ....components.theming import apply_colors, dark_mode_switch
from ....components.header import header
from ....components.link_lists import big_button_links
from ...api import FloorsAPI, models

FLOORS_API_CLIENT = None


pages_router = APIRouter()


def project_api() -> FloorsAPI:
    assert FLOORS_API_CLIENT is not None  # Initialized by mount_pages()
    return FLOORS_API_CLIENT


@ui.page(path="/", api_router=pages_router)
def floors_home():
    header("Floor Plans")
    pages = [
        [
            ("Départs/Arrivées", "sym_o_sync_alt", ""),
            ("Stats & Charts", "sym_o_query_stats", ""),
        ],
        [
            ("LBA", "sym_o_map", ""),
            ("LBB", "sym_o_map", ""),
            ("LAA", "sym_o_map", ""),
        ],
        [
            ("Admin", "sym_o_admin_panel_settings", "admin"),
            ("Settings", "sym_o_handyman", ""),
            # ("Users", "sym_o_person_apron", ''),
        ],
    ]
    big_button_links(pages)


def mount_pages(parent_router: APIRouter, api_host: str, api_port: int):
    floors_api = FloorsAPI(host=api_host, port=api_port)
    parent_router.include_router(pages_router)



#!!! YOU CAN USE THIS WITH SQLModels: from __future__ import annotations

import uuid
from typing import List, Optional, TYPE_CHECKING
from enum import Enum
import datetime

from sqlmodel import Field, Relationship
from pydantic import EmailStr, JsonValue

from .database import FloorsSQLModel


class FacilityBase(FloorsSQLModel):
    name: str = Field(unique=True, index=True, nullable=False)


class Facility(FacilityBase, table=True):
    id: uuid.UUID | None = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )
    floors: List["Floor"] = Relationship(back_populates="facility")


class FloorBase(FloorsSQLModel):
    name: str = Field(unique=True, nullable=False)

    # ???
    facility_id: Optional[uuid.UUID] = Field(
        # default=None,
        foreign_key="facility.id",
        # index=True,
        # nullable=True
    )


class Floor(FloorBase, table=True):
    id: uuid.UUID | None = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )
    facility: Facility = Relationship(back_populates="floors")
    seats: list["Seat"] = Relationship(back_populates="floor")


class SeatBase(FloorsSQLModel):
    code: str = Field(unique=True, nullable=False)
    position_x: int
    position_y: int

    # ???
    floor_id: Optional[uuid.UUID] = Field(
        # default=None,
        foreign_key="floor.id",
        # index=True,
        # nullable=True
    )


class Seat(SeatBase, table=True):
    id: uuid.UUID | None = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )

    floor: Floor = Relationship(back_populates="seats")
    assignations: list["Assignation"] = Relationship(back_populates="seat")


class ResourceCategory(str, Enum):
    USER = "User"
    WORKSTATION = "Workstation"
    SOFTWARE_LICENSE = "SoftwareLicense"
    OTHER = "Other"


class ResourceBase(FloorsSQLModel):
    category: ResourceCategory = Field(nullable=False, index=True)
    name: str
    properties: str  # JsonValue


class Resource(ResourceBase, table=True):
    id: uuid.UUID | None = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )
    assignations: list["Assignation"] = Relationship(back_populates="resource")


class AssignationStatus(str, Enum):
    PLANNED = "Planned"
    WIP = "In Progress"
    READY = "Ready"


class AssignationBase(FloorsSQLModel):
    start_date: datetime.date
    end_date: datetime.date
    status: AssignationStatus = Field(
        nullable=False,
        default=AssignationStatus.PLANNED,
        index=True,
    )

    # ???
    seat_id: Optional[uuid.UUID] = Field(
        # default=None,
        foreign_key="seat.id",
        # index=True,
        # nullable=True
    )
    resource_id: Optional[uuid.UUID] = Field(
        # default=None,
        foreign_key="resource.id",
        # index=True,
        # nullable=True
    )


class Assignation(AssignationBase, table=True):
    id: uuid.UUID | None = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )

    seat: Seat = Relationship(back_populates="assignations")
    resource: Resource = Relationship(back_populates="assignations")

    tasks: list["Task"] = Relationship(back_populates="assignation")


class TaskType(str, Enum):
    SETUP_WORKSTATION = "Setup Workstation"
    INSTALL_SOFTWARES = "Install Softwares"


class TaskStatus(str, Enum):
    PENDING = "Pending"
    IN_PROGRESS = "In Progress"
    COMPLETED = "Completed"
    BLOCKED = "Blocked"


class TaskBase(FloorsSQLModel):
    type: TaskType
    status: TaskStatus = Field(decimal_places=TaskStatus.PENDING)
    notes: str

    # ???
    assignation_id: Optional[uuid.UUID] = Field(
        # default=None,
        foreign_key="assignation.id",
        # index=True,
        # nullable=True
    )


class Task(TaskBase, table=True):
    id: uuid.UUID | None = Field(
        default_factory=uuid.uuid4, primary_key=True, index=True
    )

    assignation: Assignation = Relationship(back_populates="tasks")



from .._app import AppBackend, AppFrontend, FastAPI

from .backend import main as floors_backend_main

from .frontend import main as floors_frontend_main


class FloorsBackend(AppBackend):
    async def startup(self, app: FastAPI):
        await floors_backend_main.startup(app)

    async def teardown(self, app: FastAPI):
        await floors_backend_main.teardown(app)

    def mount(self, app: FastAPI):
        floors_backend_main.mount(app)


class FloorsFrontend(AppFrontend):
    async def startup(self, app: FastAPI):
        await floors_frontend_main.startup(app)

    async def teardown(self, app: FastAPI):
        await floors_frontend_main.teardown(app)

    def mount(self, app: FastAPI):
        floors_frontend_main.mount(app, self.settings.api_host, self.settings.api_port)



from .._app import AppFrontend, FastAPI

from .main import startup, teardown, mount


class HomeFrontend(AppFrontend):
    async def startup(self, app: FastAPI):
        await startup(app)

    async def teardown(self, app: FastAPI):
        await teardown(app)

    def mount(self, app: FastAPI):
        mount(app, self.settings.api_host, self.settings.api_port)



from typing import Generator

from sqlmodel import create_engine, SQLModel, Session, text, SQLModel, MetaData
from sqlalchemy.orm import registry

PYFOST_PROJECTS_DATABASE_URL = "sqlite:///./fost.gestion_studio.projects.db"
# For PostgreSQL (example, ensure you have python-dotenv and a .env file or set env var)
# from dotenv import load_dotenv
# import os
# load_dotenv()
# DATABASE_URL = os.getenv("DATABASE_URL", "postgresql://user:password@host:port/dbname")

# echo=True is good for development to see SQL queries
projects_engine = create_engine(PYFOST_PROJECTS_DATABASE_URL, echo=False)
projects_registry = registry()


class FloorsSQLModel(SQLModel, registry=projects_registry):
    pass


def create_db_and_tables():
    FloorsSQLModel.metadata.create_all(projects_engine)
    if projects_engine.url.drivername.lower().startswith("sqlite://"):
        with projects_engine.connect() as connection:
            connection.execute(text("PRAGMA foreign_keys=ON"))  # for SQLite only


def get_session() -> Generator[Session, None, None]:
    with Session(projects_engine) as session:
        yield session



from nicegui import ui

from ..settings import PyFostSettings

settings = PyFostSettings()


def apply_colors(dark=False):
    if dark:
        ui.colors(**settings.colors.dark.model_dump())
    else:
        ui.colors(**settings.colors.light.model_dump())


def dark_mode_switch():
    dark = ui.dark_mode()

    def switch(e):
        dark.toggle()
        apply_colors(dark=dark.value)
        if dark.value:
            e.sender.props("icon=sym_o_light_mode")
        else:
            e.sender.props("icon=sym_o_dark_mode")

    ui.button(icon="sym_o_light_mode", on_click=switch).props("fab")



from pydantic import BaseModel, Field, AnyUrl, IPvAnyNetwork
from pydantic_settings import BaseSettings, SettingsConfigDict
from ._version import version

current_version = version


class ServiceSettings(BaseModel):
    title: str
    description: str
    version: str = current_version
    host: IPvAnyNetwork = "127.0.0.1"
    port: int = 8001
    reload: bool = True


class AppBackendSettings(ServiceSettings):
    database_url: AnyUrl | None = None


class AppFrontendSettings(ServiceSettings):
    api_host: IPvAnyNetwork = "127.0.0.1"
    api_port: int = 8001
    nice_secret: str = "THIS_IS_NOT_A_SAFE_VALUE"


class AppAllSettings(AppBackendSettings, AppFrontendSettings):
    pass


class AppsSettings(BaseModel):
    home_frontend: AppFrontendSettings = AppFrontendSettings(
        title="Fost Gestion-Studio - Home",
        description="Landing page",
        port=8001,
    )

    floors_backend: AppBackendSettings = AppBackendSettings(
        title="Fost Gestion-Studio - Floors Backend",
        description="API for managing Seats assignements and Related Tasks.",
        port=8010,
    )
    floors_frontend: AppFrontendSettings = AppFrontendSettings(
        title="Fost Gestion-Studio - Floors Frontend",
        description="GUI for managing Users, Projects, and Tasks.",
        port=8011,
    )

    projects_backend: AppBackendSettings = AppBackendSettings(
        title="Fost Gestion-Studio - Projects Backend",
        description="API for managing Users, Projects, and Tasks.",
        port=8020,
    )
    projects_frontend: AppFrontendSettings = AppFrontendSettings(
        title="Fost Gestion-Studio - Projects Frontend",
        description="GUI for managing Users, Projects, and Tasks.",
        port=8021,
    )


class RuntimeSettings(BaseModel):
    backends: AppBackendSettings = AppBackendSettings(
        title="Fost Gestion-Studio - All Backends",
        description="A service with all Backend routes",
        port=8001,
    )
    frontends: AppFrontendSettings = AppFrontendSettings(
        title="Fost Gestion-Studio - All Frontends",
        description="A service with all Frontend routes",
        port=8002,
        api_port=8001,
    )
    all: AppAllSettings = AppAllSettings(
        title="Fost Gestion-Studio - All Apps",
        description="A service with all Fost apps",
        port=8090,
        api_port=8090,
    )


class ColorTheme(BaseModel):
    primary: str = "#fdc217"
    secondary: str = "#222"
    accent: str = "#9c27b0"
    dark: str = "#1d1d1d"
    dark_page: str = "oklch(27.4% 0.006 286.033)"  # zinc-800
    positive: str = "#21ba45"
    negative: str = "#c10015"
    info: str = "#31ccec"
    warning: str = "#f2c037"
    #
    box1: str = "oklch(96.7% 0.001 286.375)"  # zinc-100
    box2: str = "oklch(87.1% 0.006 286.286)"  # zinc-300
    box3: str = "oklch(55.2% 0.016 285.938)"  # zinc-500


class UIColors(BaseModel):
    dark: ColorTheme = ColorTheme(
        secondary="oklch(92% 0.004 286.32)",  # zinc-200
        box1="oklch(44.2% 0.017 285.786)",  # zinc-600
        box2="oklch(70.5% 0.015 286.067)",  # zinc-400
        box3="oklch(92% 0.004 286.32)",  # zinc-200
    )
    light: ColorTheme = ColorTheme()


class PyFostSettings(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="PYFOST__",
        # This will cummulate all the .env files,
        # so .env.DEV always wins if it exits:
        env_file=(".env", ".env.ALL", ".env.PROD", ".env.DEV"),
        env_file_encoding="utf-8",
        env_nested_delimiter="__",
        # nested_model_default_partial_update=False,
    )

    apps: AppsSettings = Field(default_factory=AppsSettings)
    runtime: RuntimeSettings = Field(default_factory=RuntimeSettings)

    colors: UIColors = Field(default_factory=UIColors)


def test():
    import os, pprint

    # os.environ["PYFOST__APPS__PROJECTS__HOST"] = "0.0.0.0"
    # os.environ["PYFOST__APPS__PROJECTS__PORT"] = "8002"

    settings = PyFostSettings()
    pprint.pprint(settings.model_dump())

    # assert settings.apps.projects.port == 8002


if __name__ == "__main__":
    test()



from __future__ import annotations

# This duplicates all the query and response models from the backend.
# Which sucks.
# But in real world situation, this api client would be generated from
# the backend openapi specs, and all would be duplicated.
# So I guess that's ok.
# (and we should generate this api from the backend openapi!!!)

import enum

from pydantic import BaseModel, Field, EmailStr, UUID4


class UserBase(BaseModel):
    id: UUID4
    email: EmailStr
    login: str


class User(UserBase):
    projects: list[Project]


class Project(BaseModel):
    id: UUID4
    code: str
    title: str


class TaskStatus(str, enum.Enum):
    NYS = "NYS"
    WIP = "WIP"
    Done = "Done"


class Task(BaseModel):
    id: UUID4
    project: Project
    title: str
    description: str | None = None
    status: TaskStatus = Field(default=TaskStatus.NYS)
    assignee_id: str | None = None
    assignee: UserBase | None = None



from contextlib import asynccontextmanager

from fastapi import FastAPI, APIRouter

from .admin import mount_admin
from .pages import mount_pages
from ...settings import PyFostSettings


async def startup(app: FastAPI):
    print("INFO:     Starting up Projects Frontend")


async def teardown(app: FastAPI):
    print("INFO:     Shutting down Projects Frontend...")


def mount(app: FastAPI, api_host: str, api_port: int):
    projects_app_frontend_router = APIRouter(prefix="/apps/projects")
    mount_pages(projects_app_frontend_router, api_host=api_host, api_port=api_port)
    mount_admin(projects_app_frontend_router, api_host=api_host, api_port=api_port)
    app.include_router(projects_app_frontend_router)


@asynccontextmanager
async def lifespan(app: FastAPI):
    yield


settings = PyFostSettings().apps.projects_frontend

app = FastAPI(
    title=settings.title,
    description=settings.description,
    version=settings.version,
    lifespan=lifespan,
)


mount(app, settings.api_host, settings.api_port)

if __name__ == "__main__":
    import uvicorn

    uvicorn.run(
        "pyfost.gestion_studio.projects.frontend.main:app",
        host=settings.host,
        port=settings.port,
        reload=settings.reload,
    )



from __future__ import annotations


from nicegui import ui

from pyfost.adminui import Admin, ModelRenderer

from ...api import FloorsAPI, models
from ....components.header import header

# class FacilityRenderer(ModelRenderer):
#     @classmethod
#     def cell_email(self, model, admin):
#         return str(model.email)

#     @classmethod
#     def cell_projects(self, model, admin):
#         return ",".join([i.code for i in model.projects])

#     @classmethod
#     def render_login(cls, model, admin):
#         ui.button(model.login)
#         # with ui.row():
#         #     for t in model.tags:
#         #         ui.button(
#         #             t.name,
#         #             on_click=lambda t=t: ui.navigate.to(
#         #                 f"{admin.prefix}/{t.__class__.__name__}/{t.name}"
#         #             ),
#         #         ).props("flat")


# class TaskRenderer(ModelRenderer):
#     @classmethod
#     def cell_project(self, model, admin):
#         return model.project.title

#     @classmethod
#     def cell_status(self, model, admin):
#         return model.status.value

#     @classmethod
#     def cell_assignee(self, model, admin):
#         assignee = model.assignee
#         if assignee is None:
#             return "<FREE>"
#         return assignee.login


def mount_admin(parent_router, api_host: str, api_port: int):
    floors_api = FloorsAPI(host=api_host, port=api_port)

    async def get_facilities() -> list[models.Facility]:
        return await floors_api.get_facilities()

    async def get_floors() -> list[models.Floor]:
        return await floors_api.get_floors()

    async def get_seats() -> list[models.Seat]:
        return await floors_api.get_seats()

    async def get_resources() -> list[models.Resource]:
        return await floors_api.get_resources()

    async def get_assignations() -> list[models.Assignation]:
        return await floors_api.get_assignations()

    async def get_tasks() -> list[models.Task]:
        return await floors_api.get_tasks()

    def admin_header():
        header(
            "Floor Plans Admin",
            back_target=parent_router.prefix + "/",
            back_icon="sym_o_arrow_circle_left",
        )

    admin = Admin("/admin")
    admin.title = "Floor Plans Admin"
    admin.set_header(admin_header)
    admin.add_view(models.Facility, get_facilities)
    admin.add_view(models.Floor, get_floors)
    admin.add_view(models.Seat, get_seats)
    admin.add_view(models.Resource, get_resources)
    admin.add_view(models.Assignation, get_assignations)
    admin.add_view(models.Task, get_tasks)
    admin.add_to_router(parent_router)



import uuid
from typing import List, Optional

from fastapi import APIRouter, Depends, HTTPException, status
from sqlmodel import Session, select

from ..database import get_session
from ..models import Task as DBTask, User as DBUser, Project as DBProject, TaskStatus

# Import Pydantic schemas
from .schemas import TaskRead, TaskCreate, TaskUpdate, UserRead, ProjectRead

router = APIRouter(
    prefix="/tasks",
    tags=["Projects Tasks"],
)


@router.post("/", response_model=TaskRead, status_code=status.HTTP_201_CREATED)
def create_task(
    *, session: Session = Depends(get_session), task_in: TaskCreate
) -> TaskRead:
    project_db = session.get(DBProject, task_in.project_id)
    if not project_db:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Project with id {task_in.project_id} not found",
        )

    assignee_db_model: Optional[DBUser] = None
    if task_in.assignee_id:
        assignee_db_model = session.get(DBUser, task_in.assignee_id)
        if not assignee_db_model:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail=f"Assignee (User) with id {task_in.assignee_id} not found",
            )

    db_task = DBTask.model_validate(task_in)  # Creates DBTask instance from TaskCreate
    session.add(db_task)
    session.commit()
    session.refresh(db_task)
    # db_task now has .assignee and .project loaded if relationships are set up (default is lazy load)

    # Construct TaskRead response
    task_data_for_resp = db_task.model_dump()  # Core fields from DBTask

    assignee_resp_model: Optional[UserRead] = None
    if (
        db_task.assignee
    ):  # Accessing .assignee will trigger lazy load if not already loaded
        assignee_resp_model = UserRead.model_validate(db_task.assignee)

    project_resp_model: ProjectRead = ProjectRead.model_validate(db_task.project)

    return TaskRead(
        **task_data_for_resp, assignee=assignee_resp_model, project=project_resp_model
    )


@router.get("/", response_model=List[TaskRead])
def read_tasks(
    *,
    session: Session = Depends(get_session),
    skip: int = 0,
    limit: int = 100,
    project_id: Optional[uuid.UUID] = None,
    assignee_id: Optional[uuid.UUID] = None,
    status: Optional[TaskStatus] = None,
) -> List[TaskRead]:
    statement = select(DBTask)
    if project_id:
        statement = statement.where(DBTask.project_id == project_id)
    if assignee_id:
        statement = statement.where(DBTask.assignee_id == assignee_id)
    if status:
        statement = statement.where(DBTask.status == status)

    db_tasks = session.exec(
        statement.offset(skip)
        .limit(limit)
        .options(
            # Eager load related objects if performance is an issue for N+1 queries
            # from sqlalchemy.orm import selectinload
            # selectinload(DBTask.assignee),
            # selectinload(DBTask.project)
        )
    ).all()
    return db_tasks

    # tasks_read_list = []
    # for task in db_tasks:
    #     task_data = task.model_dump()
    #     assignee_data = (
    #         UserRead.model_validate(task.assignee) if task.assignee else None
    #     )
    #     project_data = ProjectRead.model_validate(
    #         task.project
    #     )  # Task always has a project
    #     tasks_read_list.append(
    #         TaskRead(**task_data, assignee=assignee_data, project=project_data)
    #     )

    # return tasks_read_list


@router.get("/{task_id}", response_model=TaskRead)
def read_task(
    *, session: Session = Depends(get_session), task_id: uuid.UUID
) -> TaskRead:
    db_task = session.get(
        DBTask, task_id
    )  # Could add options for eager loading here too
    if not db_task:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Task not found"
        )
    return db_task
    # task_data = db_task.model_dump()
    # assignee_data = (
    #     UserRead.model_validate(db_task.assignee) if db_task.assignee else None
    # )
    # project_data = ProjectRead.model_validate(db_task.project)

    # return TaskRead(**task_data, assignee=assignee_data, project=project_data)


@router.patch("/{task_id}", response_model=TaskRead)
def update_task(
    *, session: Session = Depends(get_session), task_id: uuid.UUID, task_in: TaskUpdate
) -> TaskRead:
    db_task = session.get(DBTask, task_id)
    if not db_task:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Task not found"
        )

    task_data_update = task_in.model_dump(exclude_unset=True)  # TaskUpdate is SQLModel

    # Handle assignee_id separately if it's in the update and needs validation or special handling
    if "assignee_id" in task_data_update:
        new_assignee_id = task_data_update["assignee_id"]
        if new_assignee_id is not None:
            assignee_db = session.get(DBUser, new_assignee_id)
            if not assignee_db:
                raise HTTPException(
                    status_code=status.HTTP_404_NOT_FOUND,
                    detail=f"New assignee (User) with id {new_assignee_id} not found",
                )
            # db_task.assignee_id will be set by setattr below
        # else: assignee_id is None, so unassign. setattr will handle.

    for key, value in task_data_update.items():
        setattr(db_task, key, value)

    session.add(db_task)
    session.commit()
    session.refresh(db_task)

    # Construct response
    task_data_resp = db_task.model_dump()
    assignee_resp = (
        UserRead.model_validate(db_task.assignee) if db_task.assignee else None
    )
    project_resp = ProjectRead.model_validate(db_task.project)

    return TaskRead(**task_data_resp, assignee=assignee_resp, project=project_resp)


@router.delete("/{task_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_task(*, session: Session = Depends(get_session), task_id: uuid.UUID):
    task = session.get(DBTask, task_id)
    if not task:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Task not found"
        )
    session.delete(task)
    session.commit()
    return



from nicegui import ui


def big_button_links(pages: list[list[tuple[str, str, str]]]):
    """
    pages = [
        [
            ("Floor Plans", "sym_o_explore", "apps/floors/"),
            ("Projects", "sym_o_interactive_space", "apps/projects/"),
        ],
        [
            ("Settings", "sym_o_handyman", ""),
            ("API Doc", "auto_fix_high", "/docs"),
        ],
    ]
    """
    with ui.column(align_items="center").classes("p-8 w-full bg-box1"):
        for pages_group in pages:
            with ui.row().classes("bg-box2"):
                for title, icon, target in pages_group:
                    with ui.card().classes("gap-0"):
                        with ui.row().classes("w-full"):
                            ui.space()
                            with ui.link(target=target).classes("!no-underline"):
                                with ui.column(align_items="center"):
                                    ui.icon(icon, size="xl", color="primary")
                                    ui.label(title).classes("text-lg text-secondary")
                            ui.space()



import uuid
from typing import List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlmodel import Session, select, col
from sqlalchemy.exc import IntegrityError

from ..database import get_session
from ..models import (
    ResourceBase,
    Resource,
    ResourceCategory,
)


router = APIRouter(
    prefix="/resources",
    tags=["Floors Resources"],
)


@router.post("/", response_model=Resource, status_code=status.HTTP_201_CREATED)
def create_resource(
    *, session: Session = Depends(get_session), resource_in: ResourceBase
) -> Resource:
    new_resource = Resource.model_validate(resource_in)
    try:
        session.add(new_resource)
        session.commit()
        session.refresh(new_resource)
        return new_resource
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.get("/", response_model=List[Resource])
def read_resources(
    *, session: Session = Depends(get_session), skip: int = 0, limit: int = 100
) -> List[Resource]:
    resources = session.exec(select(Resource).offset(skip).limit(limit)).all()
    return resources


@router.get("/{user_id}", response_model=Resource)
def read_resource(*, session: Session = Depends(get_session), resource_id: uuid.UUID):
    resource = session.get(Resource, resource_id)
    if not resource:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Resource not found"
        )
    return resource


@router.patch("/{user_id}", response_model=Resource)
def update_resource(
    *,
    session: Session = Depends(get_session),
    resource_id: uuid.UUID,
    resource_in: ResourceBase,
) -> Resource:
    resource = session.get(Resource, resource_id)
    if not resource:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Resource not found"
        )

    data = resource_in.model_dump(exclude_unset=True)
    for key, value in data.items():
        setattr(resource, key, value)

    try:
        session.add(resource)
        session.commit()
        session.refresh(resource)
        return resource
    except IntegrityError as e:
        session.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Database integrity error: {e}",
        )


@router.delete("/{resource_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_resource(*, session: Session = Depends(get_session), resource_id: uuid.UUID):
    resource = session.get(Resource, resource_id)
    if not resource:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Resource not found"
        )

    # FIXME: Deal with related stuff (assignment, etc...)

    session.delete(resource)
    session.commit()
